      program field
C=======================================================================
C         Author : Bart Faatz
C         Company: DESY/HASYLAB
C         Date   : April 17, 2004
C         Version: 0.8
C         Filename: Field8.f
C
C
C  program to create magnetic field structures for Genesis 1.3
C-----------------------------------------------------------------------
      INCLUDE  'genesis.def'
      INCLUDE  'input.cmn'
      INCLUDE  'magnet.cmn'
c
      CHARACTER number*3, outfl*30, magouttemp*30, batchout*30
      character*30 UND,ULENGTH,STRUCT,FLength,FStrength,FOffset,
     +             SMAG,DEFTYP
      REAL*8 CORX(NZMAX),CORY(NZMAX),atmp
      REAL*8  QLEN(25),QSTR(25),XKW0, ULEN(25), DEFL, DEFAMP, RN, X
      INTEGER imz(9), i, j, k, j2, max_num, str_len,space,
     +        ierr, NQUAD,QQUAD,L,NFOC,NDRF,tmpin,
     +        NUND, LUND, IOFF, nbat
c
      str_len=3     ! Str_len=3 limits number of output to 999 files
      nbat = 26
c
c     READ in external file "parms.in"
      CALL PARMS(UND,ULENGTH,STRUCT,FLength,FStrength,FOffset,SMAG,
     +           DEFTYP,DEFL,DEFAMP)
c
c
c
      IF(((DELAW.GT.SMALL).AND.(IERTYP.NE.0)).OR.(DEFAMP.GT.SMALL)) THEN
          IF(XLAMD.NE.2.0) THEN
              write(*,*)'undulator period has to be 2 times unit length'
              STOP
	    ENDIF
      ENDIF
c
c Define Undulator structure
c
 1    i = INDEX(UND,' ')
      IF(i.EQ.1) THEN
         UND(1:29) = UND(2:30)
         GOTO 1
      ENDIF
      NUND = i-1   ! number of elements in periodic structure
c
c Define length of structure
c
      DO i=1,NUND
          IF((UND(i:i).EQ.'0').OR.(UND(i:i).EQ.'O')) UND(i:i) = '0'
      END DO
      ierr=extractval(ULength,ULEN,NUND)
      LUND=0
      IOFF = 0
      DO i=1,NZMAX
         AWZ(i) = 0.
      END DO
      RN = 0.
c
c Fill complete array with undulator
c
      DO i=1,NSEC
         DO j=1,NUND
            IF(UND(j:j).NE.'0') THEN
               DO k=LUND+1+(i-1)*IOFF,ULEN(j)+LUND+(i-1)*IOFF
                   AWZ(k) = AW0
               END DO
            ENDIF
            LUND=LUND+ULEN(j)
         END DO
         IOFF = LUND
         LUND = 0
      END DO ! loop over NSEC
      LUND = IOFF
c
c Define focusing structure
c
 2    i = INDEX(STRUCT,' ')
      IF(i.EQ.1) THEN
         STRUCT(1:29) = STRUCT(2:30)
         GOTO 2
      ENDIF
      NQUAD = i-1   ! number of elements in periodic structure
c
c Define length and strength of quads
c
      QQUAD = 0
      DO i=1,NQUAD
          IF((STRUCT(i:i).NE.'0').AND.(STRUCT(i:i).NE.'O')) THEN
               QQUAD=QQUAD+1
               ELSE
               STRUCT(i:i) = '0'
          ENDIF
      END DO
      ierr=extractval(FLength,QLEN,NQUAD)
      ierr=extractval(FStrength,QSTR,QQUAD)
      k=1
      L=0
      DO i=1,NZMAX
         QFLD(i) = 0.
      END DO
      DO i=1,NQUAD
         IF(STRUCT(i:i).NE.'0') THEN
            DO j=L+1,QLEN(i)+L
                QFLD(j) = QSTR(k)*586.0
            END DO
            k=k+1
         ENDIF
         L=L+QLEN(i)
      END DO
c
c L is length of focusing unit cell: compare undulator section 
c length with focusing section length to fill entire structure
c periodically (gaps between undulator segments)
c
      NFOC=INT(NSEC*LUND/L)
      IF(NFOC*L.LT.NSEC*LUND) NFOC = NFOC+1
      NDRF = NFOC*L-NSEC*LUND
c
c Fill complete array with quadrupoles
c
      DO i=1,NFOC-1
         DO j=1,L
                QFLD(j+i*L) = QFLD(j)
         END DO
      END DO ! loop over NSEC
c
      ierr=extractval(FOffset,QSTR,2)
      QFDX = QSTR(1)*1.E-6
      QFDY = QSTR(2)*1.E-6
c
c set some parameters
c
      DELZ = 1./XLAMD            ! needed for UNITLENGTH
      NSTEPZ = MAX(NSEC*LUND,NFOC*L)       ! total length in UNITLENGTH
      XLAMD = XLAMD*UNITLENGTH   ! real value for XLAMD
      XKW0 = TWOPI/XLAMD         ! needed for "MAGFIELD.F"
      max_num = 1
      magin = 1
c
c Add taper to the undulator field
c
      DO i=1,NSTEPZ
           atmp = AWZ(i)
           awz(i)=faw0(i,wcoefz,delz,xlamd,atmp,nstepz)
      END DO
c
      magoutfile='tmp~.lat'
      CALL MAGWRITE(CORX,CORY,XKW0)
c
c If errors are requested, do you need more than one file
c
      IF((QFDX.GT.SMALL).OR.(QFDY.GT.SMALL).OR.
     +   (DELAW.GT.SMALL).OR.(DEFAMP.GT.SMALL)) THEN
         write(*,*) 'You request errors: how many' //
     +       ' files do you want to create (<1000)? '
         read(*,*) max_num
         IF(max_num.GE.1000) THEN
            write(*,*) ' maximum number exceeded: program terminated'
            STOP
         ENDIF
         IF(max_num.GT.1) THEN
           WRITE(*,*) ' Give output file name (excluding extension): '
           READ(*,30) outfl
         ENDIF
      ENDIF
      IF(max_num.EQ.1) THEN
         WRITE(*,*) ' Give output file name: '
         READ(*,30) magouttemp
         outfl = ' '
      ENDIF
c
      space=INDEX(outfl(1:12),' ')
      outfl(space:space) = '.'
      magout=1
      maginfile = 'tmp~.lat'

      IF(DEFAMP.GT.SMALL) ISEED = -12345
      DO j=1,max_num
         CALL FILE_NUM(J,number,str_len)
c
c Fill complete array with undulator in case DEFAMP <> 0
c
      IF(DEFAMP.GT.SMALL) THEN
      maginfile = 'tmp~.lat'
      do i=1,9
         imz(i) = 0
      end do
      CALL MAGREAD(CORX,CORY,imz,0)
      DO i=1,NZMAX
         QFLD(i) = QFLD(i)*586.
      ENDDO
      LUND=0
      IOFF = 0
      DO i=1,NSEC
         DO j2=1,NUND
            IF(UND(j2:j2).NE.'0') THEN
               DO k=LUND+1+(i-1)*IOFF,ULEN(j2)+LUND+(i-1)*IOFF
                  IF(MOD(k-(LUND+1+(i-1)*IOFF),NINT(DEFL)).EQ.0)
     +            RN=DEFAMP*(2.D0*RAN1(ISEED)-1.d0)*DSQRT(3.D0) !uniform error distribution
                  IF(INDEX(DEFTYP,'constant').NE.0)AWZ(k)=AWZ(k)*(1.+RN)
                  IF(INDEX(DEFTYP,'sinus').NE.0) AWZ(k) = AWZ(k)*(1.+RN*
     +                    SIN((k-(LUND+1+(i-1)*IOFF))*TWOPI/DEFL))
                  IF(INDEX(DEFTYP,'para').NE.0) THEN
                       X = MOD(k-(LUND+1+(i-1)*IOFF),NINT(DEFL))/DEFL
			AWZ(k)=AWZ(k)*(1.+4.*RN*(X**2-X))
			ENDIF
                  IF(INDEX(DEFTYP,'saw').NE.0) THEN
                       X = MOD(k-(LUND+1+(i-1)*IOFF),NINT(DEFL))/DEFL
			AWZ(k)=AWZ(k)*(1.+X*RN)
			ENDIF
               END DO
            ENDIF
            LUND=LUND+ULEN(j2)
         END DO
         IOFF = LUND
         LUND = 0
      END DO ! loop over NSEC
c
c Correct to keep 1st and 2nd field integral zero
c
      IF(INDEX(DEFTYP,'constant').NE.0) THEN
         DO i=1,NSTEPZ,2
            IF(awz(i)*awz(i+2).GT.SMALL) THEN
               awz(i+1) = (awz(i+2)+awz(i))/2.
               ELSE
               IF(awz(i+1).GT.SMALL) awz(i+1) = awz(i)
            ENDIF
         ENDDO
      ENDIF
c
      magoutfile='tmp~.lat'
      CALL MAGWRITE(CORX,CORY,XKW0)
      ENDIF
c
c  Loop over file number
c
         outfl(space+1:space+str_len) = number(1:str_len)
         IF(max_num.GT.1) THEN
			MAGOUTFILE = outfl(1:space+str_len)
			ELSE
                  MAGOUTFILE  = magouttemp
         ENDIF
         CALL MAGFIELD(XKW0,0)
c
c The following part has been commented out.
c It can be used to generate files for batch jobs
c
cb      batchout = outfl(1:space-1)//number(1:str_len)//'.bat'
cb      OPEN(Nbat,FILE=batchout,STATUS='unknown')
cb      write(nbat,*) outfl(1:space-1)//'.in'
cb      write(nbat,*) outfl(1:space-1)//number(1:str_len)//'.out'
cb      write(nbat,*) magoutfile(1:space+str_len)
cb      close(Nbat)
	
      END DO
      open(9,file='tmp~.lat')
      close(9,status='delete')
c
      STOP
 30   FORMAT(A30)
      END                                                               
c
      SUBROUTINE FILE_NUM(filenum,strg_num,n)                           
C=======================================================================
C         Author : Bart Faatz                                           
C         Company: DESY/HASYLAB                                         
C         Date   : 22-09-97                                            
C         Version: 1.0                                                  
C         Filename: FILE_NUM.SUB                                        
C                                                                       
C     Determine file number in case of scan over different files.       
C     filenum is the file number.                                       
C                                                                       
C-----------------------------------------------------------------------
      INTEGER n, filenum, I, number                                     
      CHARACTER strg_num*12                                             
C                                                                       
      number = filenum                                                  
      DO 10 I=1,n                                                       
         power = FLOAT(10**(n-I))                                       
         strg_num(I:I) = CHAR(INT((FLOAT(number)+.5)/power)+48)         
         number = number-INT((FLOAT(number)+.5)/power)*NINT(power)      
 10   CONTINUE                                                          
C                                                                       
      RETURN                                                            
      END                                                               
c
c The following files are needed from Genesis
c In addition, "input.cmn", "io.cmn", "magnet.cmn" and "genesis.def"
c
      include "magfield.f"
      include "string.f"
      include "math.f"
c
c The following file is a part of "output.f"
c Only this sub-selection is used in order to limit the number of 
c additional files/common blocks needed
c
      include "varia.f"
c
c The following file defines the IO of "field.f" itself
c Part of the info in this file is not needed, but used by
c other application programs.
c
      include "parms.f"

